"""Command Line Editor for cards with the Librarian API."""
from __future__ import print_function
__author__ = 'Taylor "Nekroze" Lawson'
__email__ = 'nekroze@eturnilnetwork.com'
import os
import sys
from .console import Console


def clear():
    """Platform independent clear console screen."""
    if "windows" not in sys.platform.lower():
        os.system("clear")
    else:
        os.system("cls")


class CLE(Console):
    """The command line editor for cards."""
    def __init__(self, colormap, card):
        """
        Can take up to two arguments to define the code and name of the new
        card before starting the edit although it is optional.
        """
        Console.__init__(self)
        self.colormap = colormap
        self.card = card
        self.prompt = "{Csym}[{Ckey}EDIT{Csym}]>{Cres}".format(**self.colormap)

    def do_commit(self, _):
        """
        Save all changes and exit.
        """
        return -1

    def do_exit(self, _):
        """
        Exit without saving changes.
        """
        self.card = None
        return -1

    def do_code(self, args):
        """
        ``code <newcode>`` Set the card code to <newcode>.
        """
        if args:
            args = [arg.strip() for arg in args.split()]
            self.card.code = int(args[0])
        self.header()

    def do_name(self, args):
        """
        ``name <newname>`` Set the card name to <newname>.
        """
        if args:
            args = [arg.strip() for arg in args.split()]
            self.card.name = " ".join(args)
        self.header()

    def do_attribute(self, args):
        """
        ``attribute <attr>`` Add <attr> to this cards attributes.
        """
        if args:
            args = [arg.strip() for arg in args.split()]
            self.card.add_attribute(" ".join(args))
        self.header()

    def do_ability(self, args):
        """
        ``ability <phase> <abi>`` Add <abi> to this cards abilities under
        <phase>.
        """
        if args:
            args = [arg.strip() for arg in args.split()]
            if len(args) > 1:
                self.card.add_ability(args[0], " ".join(args[1:]))
        self.header()

    def do_info(self, args):
        """
        ``info <key> <data>`` Add <data> to this cards info under <key>.
        """
        if args:
            args = [arg.strip() for arg in args.split()]
            if len(args) > 1:
                self.card.set_info(args[0], " ".join(args[1:]), True)
        self.header()

    def do_delete(self, args):
        """
        ``delete <field> <key/index> <index>`` delete info from the card
        under field and if required key and/then the index. Use the display
        header to get index's.
        """
        if args:
            args = [arg.strip() for arg in args.split()]
            field = args[0]
            if field == "attribute" and len(args) >= 2:
                if len(self.card.attributes) <= int(args[1]):
                    del self.card.attributes[int(args[1])]
                return self.header()
            elif field == "attribute":
                self.card.attributes = []
                return self.header()
            elif field == "ability" and len(args) >= 3:
                if args[1] in self.card.abilities and \
                        len(self.card.attributes[args[1]]) >= int(args[2]):
                    del self.card.abilities[args[1]][int(args[2])]
                return self.header()
            elif field == "ability" and len(args) >= 2:
                if args[1] in self.card.abilities:
                    del self.card.abilities[args[1]]
                return self.header()
            elif field == "ability":
                self.card.abilities = {}
                return self.header()
            elif field == "info" and len(args) >= 3:
                if args[1] in self.card.info and \
                        len(self.card.info[args[1]]) >= int(args[2]):
                    del self.card.info[args[1]][int(args[2])]
                return self.header()
            elif field == "info" and len(args) >= 2:
                if args[1] in self.card.info:
                    del self.card.info[args[1]]
                return self.header()
            elif field == "info":
                self.card.info = {}
                return self.header()
        self.header()

    def header(self):
        """Display a header of card information."""
        clear()
        if self.card is None:
            return None
        card = self.card
        print("{Cval}{0}{Csym}: {Cval}{1}".format(card.code, card.name,
                                                  **self.colormap))

        print("{Csym}:::::{Ckey}Attributes".format(**self.colormap))
        for index, attr in enumerate(self.card.attributes):
            print("{Csym}[{Ckey}{0}{Csym}]{Ckey}{1}".format(str(index), attr,
                                                            **self.colormap))

        print("{Csym}:::::{Ckey}Abilities".format(**self.colormap))
        for phase, abilities in self.card.abilities.items():
            print("{Cval}{0}".format(phase, **self.colormap))
            for index, ability in enumerate(abilities):
                print("{Csym}|_____[{Ckey}{0}{Csym}]{Cval}{1}".format(
                    str(index), ability, **self.colormap))

        print("{Csym}:::::{Ckey}Info".format(**self.colormap))
        for key, value in self.card.info.items():
            print("{Cval}{0}".format(key, **self.colormap))
            for index, info in enumerate(value):
                print("{Csym}|_____[{Ckey}{0}{Csym}]{Cval}{1}".format(
                    str(index), info, **self.colormap))

    def preloop(self):
        """Display header at start."""
        Console.preloop(self)
        self.header()



"""Command Line Interface for the Librarian API."""
from __future__ import print_function
__author__ = 'Taylor "Nekroze" Lawson'
__email__ = 'nekroze@eturnilnetwork.com'
import os
import sys
from colorama import Fore
from colorama import Style
from librarian.library import Library
from librarian.card import Card
from .console import Console
from .cle import CLE


def readinput(prefix):
    """Python version independent input reading."""
    if sys.version_info > (3, 0):
        return input(prefix)
    else:
        return raw_input(prefix)


def clear():
    """Platform independent clear console screen."""
    if "windows" not in sys.platform.lower():
        os.system("clear")
    else:
        os.system("cls")


class CLI(Console):
    """The command line inteface."""
    def __init__(self, dbname=None):
        """
        Takes a filename for the database and will create it and any required
        tables if the database filename doesnt exist.
        """
        Console.__init__(self)
        self.colormap = dict(Cval=Fore.YELLOW, Csym=Fore.GREEN, Ckey=Fore.CYAN,
                             Cres=Style.RESET_ALL)
        self.prompt = "{Csym}[{Ckey}HOME{Csym}]>{Cres}".format(**self.colormap)
        dbname = "library.lbr" if dbname is None else dbname
        self.library = Library(dbname)
        if not os.path.exists(dbname):
            self.library.create_db()
        self.list_max = 50

    def do_edit(self, args):
        """Edit a card. Can take a card code to edit."""
        code = int(args) if args else 0
        card = self.library.load_card(code, cache=False)
        if card is None:
            card = Card(code)

        cle = CLE(self.colormap, card)
        cle.cmdloop()
        card = cle.card

        if card is None:
            return clear()

        self.library.save_card(card)
        clear()

    def do_delete(self, args):
        """Delete a card by code."""
        if args:
            code = int(args)
        else:
            clear()
            print("Input code to delete.")
            code = int(readinput("|>"))

        with self.library.connection() as libdb:
            libdb.execute("DELETE from CARDS where code = {0}".format(code))

    def do_list(self, args):
        """List all stored cards. Can search by a code prefix."""
        code = args if args else None
        results = self.library.filter_search(code=code)

        if not len(results):
            print("No cards could be found")
            return None

        if len(results) > self.list_max:
            results = results[:self.list_max]

        for codename in results:
            print("{Cval}{0}{Csym}: {Cval}{1}".format(*codename,
                                                      **self.colormap))

    def do_setting(self, _):
        """Change settings for this session."""
        print("Maximum cards to return from list or search.")
        self.list_max = int(readinput("|>"))

    def do_search(self, _):
        """
        Perform an advanced card search. Input nothing to skip filter or '*'
        for an info or ability as a wildcard.
        """
        code = readinput("Code\n|>")
        if not code:
            code = None
        name = readinput("Name\n|>")
        if not name:
            name = None

        abilities = {}
        while True:
            print("Ability:")
            phase = readinput("Phase\n|>")
            if not phase:
                break
            ability = readinput("Ability\n|>")
            if not ability:
                ability = '*'
            abilities[phase] = ability
        if not abilities:
            abilities = None

        attributes = []
        while True:
            attrib = readinput("Attribute\n|>")
            if not attrib:
                break
            attributes.append(attrib)
        if not attributes:
            attributes = None

        info = {}
        while True:
            print("Info:")
            key = readinput("Key\n|>")
            if not key:
                break
            value = readinput("Value\n|>")
            if not value:
                value = '*'
            info[key] = value
        if not info:
            info = None

        results = self.library.filter_search(code, name, abilities, attributes,
                                             info)
        if not len(results):
            print("No cards could be found")
            return None

        if len(results) > self.list_max:
            results = results[:self.list_max]

        for codename in results:
            print("{Cval}{0}{Csym}: {Cval}{1}".format(*codename,
                                                      **self.colormap))



"""Console super class for a Command Line Interface."""
from __future__ import print_function
__author__ = 'Taylor "Nekroze" Lawson'
__email__ = 'nekroze@eturnilnetwork.com'
import cmd
try:
    import readline
except ImportError:
    readline = None


class Console(cmd.Cmd):
    """Console is a CLI component based on the cmd module."""
    def __init__(self):
        """
        Takes a filename for the database and will create it and any required
        tables if the database filename doesnt exist.
        """
        cmd.Cmd.__init__(self)
        self.prompt = "=>> "
        if readline is None:
            self.completekey = None

    def do_hist(self, _):
        """Print a list of commands that have been entered"""
        print(self._hist)

    def do_exit(self, _):
        """Exits from the console"""
        return -1

    def do_help(self, args):
        """
        Get help on commands
        'help' or '?' with no arguments prints a list of commands for which
        help is available 'help <command>' or '? <command>' gives help on
        <command>
        """
        cmd.Cmd.do_help(self, args)

    def preloop(self):
        """
        Initialization before prompting user for commands. Despite the claims
        in the Cmd documentaion, Cmd.preloop() is not a stub.
        """
        cmd.Cmd.preloop(self)
        self._hist = []

    def postloop(self):
        """Take care of any unfinished business."""
        cmd.Cmd.postloop(self)

    def precmd(self, line):
        """
        Called after a line has been input but before it has been interpreted.
        """
        self._hist.append(line.strip())
        return line

    def postcmd(self, stop, line):
        """
        If you want to stop the console, return something that evaluates to
        true. If you want to do some post command processing, do it here.
        """
        return stop

    def emptyline(self):
        """Do nothing on empty input line"""
        self.do_help(None)

    def default(self, line):
        """
        Called on an input line when the command prefix is not recognized.
        In that case we execute the line as Python code.
        """
        self.do_help(None)



"""Command line driver entry point."""
from __future__ import print_function
__version__ = "0.1.0"
__author__ = 'Taylor "Nekroze" Lawson'
__email__ = 'nekroze@eturnilnetwork.com'
from .cli import CLI
import colorama
import argparse
import sys


def main():
    """Drive the usage of curator-cli."""
    parser = argparse.ArgumentParser(
        description="library access from the command line.")
    parser.add_argument("-v", "--version", help="Display curator-cli version",
                        action="store_true", default=False)
    parser.add_argument("--libname", help="Path to the librarian library file",
                        type=str, default=None)
    args = parser.parse_args()

    if args.version:
        print('curator-cli v'+__version__)
        sys.exit(0)

    colorama.init()
    CLI(args.libname).cmdloop()



#!/usr/bin/python
import re
import os
import sys
import platform
from setuptools import setup


__version__ = '0.1.0'
__author__ = 'Taylor "Nekroze" Lawson'
__email__ = 'nekroze@eturnilnetwork.com'
SOURCE = 'curator'
TESTDIR = 'test'
PROJECTNAME = 'librarian-curator'
PROJECTSITE = 'nekroze.eturnilnetwork.com'
PROJECTDESC = 'None'
PROJECTLICENSE = 'MIT'
PLATFORMS = ['any']

kwds = {}
kwds['version'] = __version__
kwds['description'] = PROJECTDESC
kwds['long_description'] = open('README.rst').read()
kwds['license'] = PROJECTLICENSE


setup(
    name=PROJECTNAME,
    author=__author__,
    author_email=__email__,
    url=PROJECTSITE,
    platforms=PLATFORMS,
    packages=[SOURCE],
    install_requires=['colorama>=0.2.5', 'librarian>=0.2.0'],
    entry_points={
        'console_scripts': [
            'curator = curator.driver:main',
            ]
    },
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Games/Entertainment",
        "Topic :: Utilities"
    ],
    **kwds
)
